"use strict";
this.name = "StationDockControl_EscortLaunch";
this.author = "phkb";
this.copyright = "2015 phkb";
this.description = "Script to attach to a ship after launch to get it to escort a mothership";
this.license = "CC BY-NC-SA 3.0";

/*
	This script is attached to the mothership and escort ships. occasionally a mothership will launch well ahead of some of its escorts
	(due to docking queue pressure). When that happens, a mothership can enter hyperspace before an escort even launches.
	This script will attach escorts to the mothership in one of two ways:
	1. If the mothership launches ahead of the escort, the mothership will use the "this._escorts" array to attach the escorts
	2. As a backup, the escort itself will attempt to attach itself after launch
	Hopefully this will cover all scenarios for escort launches and ensure no escort is left behind
*/

//-------------------------------------------------------------------------------------------------------------
// add an escort to the list
this.$addEscort = function $addEscort(escortShip) {
	if (this.ship.script._done) return false;
	if (!this.ship.script._escorts) this.ship.script._escorts = [];
	this.ship.script._escorts.push(escortShip);
	return true;
}

//-------------------------------------------------------------------------------------------------------------
this.$removeEscort = function $removeEscort(escortShip) {
	if (!this.ship.script._escorts) this.ship.script._escorts = [];
	for (var i = this.ship.script._escorts.length - 1; i >= 0; i--) {
		if (this.ship.script._escorts[i] === escortShip) {
			this.ship.script._escorts.splice(i, 1);
		}
	}
}

//-------------------------------------------------------------------------------------------------------------
this.npc_shipDied = function npc_shipDied(whom, why) {
	if (this.ship.script.$sdc_hold_shipDied) this.ship.script.$sdc_hold_shipDied(whom, why);
	if (this.ship.script._timer && this.ship.script._timer.isRunning) this.ship.script._timer.stop();
}

//-------------------------------------------------------------------------------------------------------------
this.npc_shipWillEnterWormhole = function npc_shipWillEnterWormhole() {
	if (this.ship.script.$sdc_hold_shipWillEnterWormhole) this.ship.script.$sdc_hold_shipWillEnterWormhole();
	if (this.ship.script._timer && this.ship.script._timer.isRunning) this.ship.script._timer.stop();
}

//-------------------------------------------------------------------------------------------------------------
// used for mothership and escort ship launches. One of these should fire and cancel the other out.
this.npc_shipLaunchedFromStation = function npc_shipLaunchedFromStation(station) {
	var s = this.ship;
	if (s.script.$sdc_hold_shipLaunchedFromStation) s.script.$sdc_hold_shipLaunchedFromStation(station);

	var ret = false;
	// pull down the debug settings from the master function
	var w = worldScripts.StationDockControl;
	var debug = w._debug;
	var l_debug = w._logLaunchType;

	if (s.script._mothership && s.script._done === false) {
		// type 1 - escort launch
		if (debug && l_debug >= 2) log(this.name, s.displayName + ": escort launch successful");

		// offer to escort the mothership
		if (s.script._mothership && s.script._mothership.isValid && s.script._mothership.status === "STATUS_IN_FLIGHT") {
			ret = s.offerToEscort(s.script._mothership);
			if (debug && l_debug >= 2) log(this.name, s.displayName + ": found leader (" + s.script._mothership.displayName + ") - offer to escort: " + ret);
		}

		// remove this ship from the mothership launch array if it was successfully added
		if (ret === true) {
			if (s.script._mothership.script.$removeEscort) s.script._mothership.script.$removeEscort(s);
		}

		// report on any errors
		if (ret === false) {
			if (!s.script._mothership) log(this.name, "!!ERROR: Mothership is null");
			if (s.script._mothership && s.script._mothership.isValid === false) log(this.name, "!!ERROR: Mothership is invalid");
			if (s.script._mothership && s.script._mothership.isValid && s.script._mothership.status !== "STATUS_IN_FLIGHT") {
				if (s.script._mothership.status !== "STATUS_LAUNCHING") log(this.name, "!!ERROR: Mothership is not in flight: " + s.script._mothership.status);
				if (s.script._mothership.status === "STATUS_LAUNCHING") {
					if (debug && l_debug >= 2) log(this.name, "!!NOTE: Mothership is launching -- setting timer to await launch");
					if (s.script._timer && s.script._timer.isRunning) s.script._timer.stop();
					s.script._timer = new Timer(this, s.script.$attachEscort, 2, 0);
					return;
				}
			}
			if (s.status === "STATUS_IN_FLIGHT" && s.script._mothership && s.script._mothership.isValid && s.script._mothership.status === "STATUS_IN_FLIGHT") {
				if (debug && l_debug >= 2) log(this.name, "!!NOTE: Escort and mothership are in flight -- setting timer to try again");
				if (s.script._timer && s.script._timer.isRunning) s.script._timer.stop();
				s.script._timer = new Timer(this, s.script.$attachEscort, 2, 0);
				return;
			}

			log(this.name, "!!ERROR: Ship launch script failed to escort mothership! " + (s.script._mothership && s.script._mothership.isValid ? "(Mthr: " + s.script._mothership.displayName + " (" + s.script._mothership.status + "), " : "") + "This: " + s.displayName + " (" + s.status + ")");
		}

	} else if (s.script._escorts && s.script._escorts.length > 0 && s.script._done === false) {
		// type 2 - mothership launch
		if (debug && l_debug >= 2) log(this.name, s.displayName + ": mothership launch successful");

		if (s.isValid && s.status === "STATUS_IN_FLIGHT") {
			if (!s.script._escorts) s.script._escorts = [];
			for (var i = 0; i < s.script._escorts.length; i++) {
				ret = s.script._escorts[i].offerToEscort(s);
				if (debug && l_debug >= 2) log(this.name, s.script._escorts[i].displayName + ": found leader (" + s.displayName + ") - offer to escort: " + ret);
				// turn off the escorts's individual launch script
				if (ret === true) {
					s.script._escorts[i].script._done = true;
				}
				// report on any errors
				if (ret === false) {
					if (s && s.status === "STATUS_LAUNCHING") {
						if (debug && l_debug >= 2) log(this.name, "!!NOTE: Mothership is still launching -- setting timer to try again");
						if (s.script._timer && s.script._timer.isRunning) s.script._timer.stop();
						s.script._timer = new Timer(this, s.script.$attachEscort, 2, 0);
						return;
					}
					if (s && s.isValid && s.status !== "STATUS_IN_FLIGHT") log(this.name, "!!ERROR: Mothership is not in flight: " + s.status);
					log(this.name, "!!ERROR: Mothership launch script failed to add escort! " + s.displayName + " (" + s.status + ", " + s.script._escorts.length + "), Escort: " + s.script._escorts[i].displayName + " (" + s.script._escorts[i].status + ")");
				}
			}
		}
	}

	s.script._done = true;

	// cleanup
	s.script._mothership = null;
	if (s.script._escorts) s.script._escorts.length = 0;
	delete s.script._timer;
	delete s.shipLaunchedFromStation;
	if (s.script.$sdc_hold_shipLaunchedFromStation) s.script.shipLaunchedFromStation = $sdc_hold_shipLaunchedFromStation;
}

//-------------------------------------------------------------------------------------------------------------
this.$attachEscort = function $attachEscort() {
	var s = this.ship;
	var ret = false;
	var w = worldScripts.StationDockControl;
	var debug = w._debug;
	var l_debug = w._logLaunchType;

	if (!s || s.isValid === false) return;
	if (s.script._done === true) return;

	if (s.script._mothership && s.script._mothership.isValid && s.script._mothership.status === "STATUS_IN_FLIGHT") {
		ret = s.offerToEscort(s.script._mothership);
		if (debug && l_debug >= 2) log(this.name, s.displayName + ": (timer) found leader (" + s.script._mothership.displayName + ") - offer to escort: " + ret);
	}
	if (s.script._escorts && s.script._escorts.length > 0 && s.status === "STATUS_IN_FLIGHT") {
		for (var i = 0; i < s.script._escorts.length; i++) {
			ret = s.script._escorts[i].offerToEscort(s);
			if (debug && l_debug >= 2) log(this.name, s.script._escorts[i].displayName + ": (timer) found leader (" + s.displayName + ") - offer to escort: " + ret);
			// turn off the escorts's individual launch script
			if (ret === true) {
				s.script._escorts[i].script._done = true;
			}
			// report on any errors
			if (ret === false) {
				if (s && s.status === "STATUS_LAUNCHING") {
					if (debug && l_debug >= 2) log(this.name, "!!NOTE: Escorts still launching -- setting timer to try again");
					if (s.script._timer && s.script._timer.isRunning) s.script._timer.stop();
					s.script._timer = new Timer(this, s.script.$attachEscort, 2, 0);
				}
				if (s && s.isValid && s.status !== "STATUS_IN_FLIGHT") log(this.name, "!!ERROR: Mothership is not in flight: " + s.status);
				//log(this.name, "!!ERROR: Mothership launch script failed to add escort! " + s.displayName + " (" + s.status + ", " + s.script._escorts.length + "), Escort: " + s.script._escorts[i].displayName + " (" + s.script._escorts[i].status + ")");
			}
		}
	}

	if (ret === false) {
		if (!s.script._timerCount) s.script._timerCount = 0;
		s.script._timerCount += 1;
		if (s.script._timerCount < 10) {
			delete s.script._timer;
			s.script._timer = new Timer(this, s.script.$attachEscort, 2, 0);
		}
		if (s.script._mothership) {
			if (debug && l_debug >= 2) log(this.name, s.displayName + ": (timer) awaiting mothership launch - try " + s.script._timerCount);
		} else {
			if (debug && l_debug >= 2) log(this.name, s.displayName + ": (timer) awaiting mothership/escorts launch - try " + s.script._timerCount);
		}
	} else {
		s.script._done = true;
		// cleanup
		s.script._mothership = null;
		if (s.script._escorts) s.script._escorts.length = 0;
	}
}