"use strict";
this.name = "StationDockControl_Populator";
this.author = "phkb";
this.copyright = "2015 phkb";
this.description = "Overrides the systemWillRepopulate with a new one that allows the station dock control to launch ships from stations.";
this.license = "CC BY-NC-SA 3.0";

this._groupContent = []; // array of ships and groups so we can keep track of group info when ships dock
this._groupCount = 1; // a counter to get added to group names for uniqueness
this._SDC = null; // object ref to main SDC system
this._debug = false; // controls output of information to the log. Slaved to StationDockControl._debug
this._logPopulator = 0; // controls level of information output to log. Slaved to StationDockControl._logPopulatorType

/*
	This script file replaces the "systemWillRepopulate" function of the standard built in populator.
	Essentially, all the "outbound" routines are removed for traders, shuttles, pirates, assassins and hunters.
	Police launches and Aegis raiders are unaffected.
	All outbound launches are controlled via the stationdockcontrol worldscript.

	We also make a list of all groups and group members here. The reason for this is that when ships dock with a station, generally all group
	information is dropped or missing from the ships at the point of docking, which means we can't tell what group they're in when they dock.
	So, we need to keep track of groups and group members outside of the ship group mechanism so we can reapply ship groups to the docking array.
*/

//-------------------------------------------------------------------------------------------------------------
this.startUpComplete = function () {

	this._SDC = worldScripts.StationDockControl;
	this._debug = this._SDC._debug;
	this._logPopulator = this._SDC._logPopulatorType;

	if (this._SDC._disable === false) {
		// replace the original repopulating function with our new one.
		worldScripts["oolite-populator"].systemWillRepopulate = this.$stationdockcontrol_systemWillRepopulate;
	} else {
		delete this.shipSpawned;
		delete this.shipDied;
		delete this.shipWillEnterWitchspace;
	}

	delete this.startUpComplete;
}

//-------------------------------------------------------------------------------------------------------------
// this handles the process of updating group names to make them unique
// this enables the dock controller to see the whole group when docked
this.shipSpawned = function (ship) {
	if (ship.group) {
		if (!ship.isPolice && !ship.isStation && !ship.isThargoid && ship.isPiloted) {
			if (ship.group.name && ship.group.name.indexOf("-") === -1) {
				ship.group.name = ship.group.name + "-" + this._groupCount;
				this._groupCount += 1;
			}
			if (!ship.group.name) {
				ship.group.name = "unknown group-" + this._groupCount;
				this._groupCount += 1;
			}
			if (ship.group.count > 1) {
				if (this._debug && this._logPopulator >= 2) log(this.name, "Group '" + ship.group.name + "', ship: " + ship.displayName);
				this.$logShipGroup(ship.group.name, ship, (ship.group.leader === ship ? true : false));
			}
		}
	}
}

//-------------------------------------------------------------------------------------------------------------
this.shipWillEnterWitchspace = function (cause, destination) {
	// clear out the array
	this._groupContent.length = 0;
}

//-------------------------------------------------------------------------------------------------------------
// adds a ship group and leader combination to the array
this.$logShipGroup = function $logShipGroup(groupName, ship, leader) {
	for (var i = 0; i < this._groupContent.length; i++) {
		if (this._groupContent[i].groupName === groupName && this._groupContent[i].ship === ship) return;
	}
	this._groupContent.push({
		groupName: groupName,
		ship: ship,
		leader: leader
	});
}

//-------------------------------------------------------------------------------------------------------------
// removes a ship from the data
this.$removeShip = function $removeShip(ship) {
	if (this._debug && this._logPopulator >= 2) log(this.name, "Removing ship: " + ship.displayName);
	for (var i = this._groupContent.length - 1; i >= 0; i--) {
		if (this._groupContent[i].ship === ship) {
			// remove the item from the array
			this._groupContent.splice(i, 1);
		}
	}
}

//-------------------------------------------------------------------------------------------------------------
// gets the group leader for a particular group
this.$getGroupLeader = function $getGroupLeader(groupName) {
	for (var i = 0; i < this._groupContent.length; i++) {
		if (this._groupContent[i].groupName === groupName && this._groupContent[i].leader === true) return this._groupContent[i].ship;
	}
	return null;
}

//-------------------------------------------------------------------------------------------------------------
// get the group name attached to a particular ship
this.$getGroupName = function $getGroupName(ship) {
	for (var i = 0; i < this._groupContent.length; i++) {
		if (this._groupContent[i].ship === ship) return this._groupContent[i].groupName;
	}
	return "";
}

//-------------------------------------------------------------------------------------------------------------
// function responsible for replenishing system contents
this.$stationdockcontrol_systemWillRepopulate = function $stationdockcontrol_systemWillRepopulate() {

	// if main station or planet is missing, something odd has
	// happened, so stop repopulation
	if (system.sun.isGoingNova || !system.mainStation || !system.mainPlanet) {
		return;
	}
	/* repopulate incoming traffic */

	// traders
	if (Math.random() < this.$repopulatorFrequencyIncoming.traderFreighters) {
		this._debugR("Incoming freighter");
		this._addFreighter(this._wormholePos());
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.traderCouriers) {
		this._debugR("Incoming courier");
		if (Math.random() < 0.5) {
			this._addCourierShort(this._wormholePos());
		} else {
			this._addCourierLong(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.traderSmugglers) {
		this._debugR("Incoming smuggler");
		this._addSmuggler(this._wormholePos());
	}

	// pirates
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateLightPacks) {
		if (system.countShipsWithPrimaryRole("pirate-light-freighter") < 6) {
			this._debugR("Incoming light pirate");
			this._addLightPirateRemote(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateLightPacksReturn) {
		if (system.countShipsWithPrimaryRole("pirate-light-freighter") < 6) {
			this._debugR("Returning light pirate");
			this._addLightPirateReturn(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateMediumPacks) {
		if (system.countShipsWithPrimaryRole("pirate-medium-freighter") < 4) {
			this._debugR("Incoming medium pirate");
			this._addMediumPirateRemote(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateMediumPacksReturn) {
		if (system.countShipsWithPrimaryRole("pirate-medium-freighter") < 4) {
			this._debugR("Returning medium pirate");
			this._addMediumPirateReturn(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateHeavyPacks) {
		if (system.countShipsWithPrimaryRole("pirate-heavy-freighter") < 2) {
			this._debugR("Incoming heavy pirate");
			this._addHeavyPirateRemote(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.pirateHeavyPacksReturn) {
		if (system.countShipsWithPrimaryRole("pirate-heavy-freighter") < 2) {
			this._debugR("Returning heavy pirate");
			this._addHeavyPirateReturn(this._wormholePos());
		}
	}

	// hunters
	if (Math.random() < this.$repopulatorFrequencyIncoming.hunterMediumPacks) {
		if (system.countShipsWithPrimaryRole("hunter-medium") < 5) {
			this._debugR("Incoming medium hunter");
			this._addMediumHunterRemote(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.hunterMediumPacksReturn) {
		if (system.countShipsWithPrimaryRole("hunter-medium") < 5) {
			this._debugR("Returning medium hunter");
			this._addMediumHunterReturn(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.hunterHeavyPacks) {
		if (system.countShipsWithPrimaryRole("hunter-heavy") < 3) {
			this._debugR("Incoming heavy hunter");
			this._addHeavyHunterRemote(this._wormholePos());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.hunterHeavyPacksReturn) {
		if (system.countShipsWithPrimaryRole("hunter-heavy") < 3) {
			this._debugR("Returning heavy hunter");
			this._addHeavyHunterReturn(this._wormholePos());
		}
	}

	// thargoids (do not appear at normal witchpoint)
	if (Math.random() < this.$repopulatorFrequencyIncoming.thargoidScouts) {
		this._debugR("Incoming thargoid scout");
		this._addThargoidScout(system.locationFromCode("TRIANGLE"));
	}
	if (Math.random() < this.$repopulatorFrequencyIncoming.thargoidStrikes) {
		this._debugR("Incoming thargoid strike force");
		this._addThargoidStrike(system.locationFromCode("TRIANGLE"));
	}

	if (Math.random() < this.$repopulatorFrequencyOutgoing.pirateAegisRaiders) {
		this._addAegisRaiders();
	}

	// police
	if (Math.random() < this.$repopulatorFrequencyOutgoing.policePacks) {
		if (system.countShipsWithPrimaryRole("police") < 30) {
			this._debugR("Launching police patrol");
			this._addPolicePatrol(this._policeLaunch());
		}
	}
	if (Math.random() < this.$repopulatorFrequencyOutgoing.policeInterceptors) {
		if (system.countShipsWithPrimaryRole("police-witchpoint-patrol") < 30) {
			this._debugR("Launching police interception patrol");
			this._addInterceptors(this._policeLaunch());
		}
	}

}